import { html, css } from 'lit-element';
import { PageViewElement } from './page-view-element.js';
import { connect } from 'pwa-helpers/connect-mixin.js';
import { store } from '../store.js';
import { SharedStyles } from './shared-styles.js';

import { appurl } from '../helpers/url.js';
import { fixDrawerPosition } from '../helpers/position-fix.js';
import { fetchResource } from '../actions/server-fetch.js';
import { addAllInjuries, updateTitle, archiveInjury, unarchiveInjury } from '../actions/injuries.js';

import injuries from '../reducers/injuries.js';
store.addReducers({
  injuries
});

import './my-injury.js';

/*
  Overview of all injuries for an athlete.
  Distinguishes between active and archived injuries, placing the latter on the bottom.
*/
class MyInjuries extends connect(store)(PageViewElement) {
  static get properties(){
    return {
      _drawerOpened: { type: Boolean },
      _loggedIn: { type: Boolean },
      _userType: { type: Number },
      _userID: { type: Number },
      _injuriesActive: { type: Array },
      _injuriesArchived: { type: Array },
      _fetchWasEmpty: { type: Boolean }
    };
  }

  static get styles() {
    return [
      SharedStyles,
      css`
      .page-content-container {
        margin-top: 15px;
      }
      `
    ];
  }

  render() {
    fixDrawerPosition(this);
    return html`
    <div id="page" class="page-content-container">
      ${this._loggedIn && this._userType === 1 ? html`
        ${this._injuriesActive.length === 0 && this._injuriesArchived.length === 0 ? html`<p>Ingen skader registrert</p>` : html ``}
        ${this._injuriesActive.map((i) => html`
          <my-injury .injury="${i}" @archived="${this._archiveInjury}" @title-changed="${this._updateTitle}"></my-injury>
        `)}
        ${this._injuriesArchived.length !== 0 ? html`<p>Arkiverte skader:</p>` : html``}
        ${this._injuriesArchived.map((i) => html`
          <my-injury .injury="${i}" @unarchived="${this._unarchiveInjury}" @title-changed="${this._updateTitle}"></my-injury>
        `)}
      ` : html`<p>Du er ikke logget inn, eller har ikke tilgang til denne siden.</p>`}
    </div>
    `
  }

  stateChanged(state){
    this._drawerOpened = state.app.drawerOpened;
    this._loggedIn = state.session.loggedIn;
    this._userType = state.session.userType;
    this._userID = state.session.userID;
    this._injuriesActive = state.injuries.injuries_active;
    this._injuriesArchived = state.injuries.injuries_archived;
  }

  constructor() {
    super();
    this._injuriesActive = [];
    this._injuriesArchived = [];
    this._fetchWasEmpty = false;
  }

  /*
    Three custom event handlers for updating an injury.
    The injury object is sent in event.detail.
  */

  _archiveInjury(event) {
    store.dispatch(archiveInjury(event.detail));
  }

  _unarchiveInjury(event) {
    store.dispatch(unarchiveInjury(event.detail));
  }

  _updateTitle(event) {
    store.dispatch(updateTitle(event.detail));
  }

  /*
    shouldFetch() is implemented because when logging in as multiple users consecutively in one browser session,
    firstUpdated() only runs once on named pages such as this, resulting in one user being able to see another user's information
    until they refresh the page. So we need to run the fetch code somewhere else.
  */
  shouldFetch() {
    const state = store.getState().injuries;
    return (!this._fetchWasEmpty && this._loggedIn && state.injuries_active.length === 0 && state.injuries_archived.length === 0);
  }

  // Fetch is done in updated() instead of firstUpdated() and wrapped in a shouldFetch() check.
  updated(changedProps) {
    if (this.shouldFetch()) {
      const url = appurl + "/api/injuries/" + this._userID;
      const fetchOptions = {
        method: "GET"
      };

      store.dispatch(fetchResource(url, fetchOptions)).then(() => {
        const data = store.getState().resources.data;
        if (data.status === 'OK') {
          this._injuriesActive = data.injuries_active;
          this._injuriesArchived = data.injuries_archived;
          if (data.injuries_active.length === 0 && data.injuries_archived.length === 0) {
            this._fetchWasEmpty = true;
          }
          store.dispatch(addAllInjuries(data.injuries_active, data.injuries_archived));
        }
      });
    }
    if (changedProps.has('_loggedIn') && !this._loggedIn) {
      this._fetchWasEmpty = false;  // Reset on logout
    }
  }
}

window.customElements.define('my-injuries', MyInjuries);
