import { LitElement, html, css } from 'lit-element';
import { SharedStyles } from './shared-styles.js';
import { connect } from 'pwa-helpers/connect-mixin.js';
import { store } from '../store.js';
import { fetchResource } from '../actions/server-fetch.js';

import { appurl } from '../helpers/url.js';
import { formDataToJson } from '../helpers/form-data.js';
import { editIcon, archiveIcon, unarchiveIcon, doneIcon } from './my-icons.js';

/*
  A single injury in an athlete's list of injuries.
  The athlete user can edit its title and toggle its archived state.
*/
class MyInjury extends connect(store)(LitElement) {
  static get properties(){
    return {
      injury: { type: Object },
      _editing: { type: Boolean }
    };
  }

  constructor() {
    super();
    this.injury = {};
    this._editing = false;
  }

  static get styles() {
    return [
      SharedStyles,
      css`
        .injury-header {
          padding: 12px 0;
        }

        .button-container {
          margin: 0 3px 3px 3px;
        }

        .injury-header, .button-container {
          display: flex;
          flex-direction: row;
          justify-content: space-between;
          align-items: center;
        }

        .injury-header-container {
          display: flex;
          align-items: flex-start;
        }

        p {
          text-align: left;
          margin-left: 12px;
          line-height: 1.3em;
        }

        .date {
          font-size: 14px;
          color: #777;
          margin-right: 12px;
          align-self: center;
        }

        input {
          border: none;
          background: none;
          font-size: 20px;
          color: var(--text-color);
          margin-top: 3px;
          margin-left: 12px;
        }

        input:-webkit-autofill,
        input:-webkit-autofill:hover,
        input:-webkit-autofill:focus,
        input:-webkit-autofill:active  {
            -webkit-box-shadow: 0 0 0 30px var(--light-sand-color) inset !important;
        }

        .action-button {
          fill: #777;
          background: none;
          border: none;
          cursor: pointer;
          margin: 0 2px;
          padding: 4px 5px;
        }
      `
    ];
  }

  render() {
    return html`
    <div class="light-background-field">
      <div class="injury-header" @click="${this._toggleDetails}">
        <form id="injuryEdit">
          <input ?disabled="${!this._editing}" id="title-input" type="text" name="title" value="${this.injury.title}" size="12">
        </form>
        <div class="injury-header-container">
          <button form="injuryEdit" id="submit-edit" style="visibility:hidden;" class="action-button"
          @click="${this._editTitle}">${doneIcon}</button>
          <div class="date">${this.injury.date_reg}</div>
        </div>
      </div>
      <div id="injury-details" style="display:none;">
        <p>Informasjon gitt av helsepersonell, behandling etc. ${this.injury.description}</p>
        <!-- TODO Add info from TreatmentPlan -->
        <p>Oppstått: ${this.injury.date_inj}</p>
        <p>Forventet frisk: ${this.injury.date_rec}</p>
        <div class="button-container">
          ${this.injury.archived ? html`
            <button class="action-button" @click="${this._unarchiveButtonClicked}">${unarchiveIcon}</button>
          ` : html`
            <button class="action-button" @click="${this._archiveButtonClicked}">${archiveIcon}</button>
          `}
          <button class="action-button" @click="${this._editButtonClicked}">${editIcon}</button>
        </div>
      </div>
    </div>
    `
  }

  _toggleDetails() {
    const details = this.shadowRoot.getElementById("injury-details");
    if (!this._editing) {
      (details.style.display == "none") ? details.style.display = "block" : details.style.display = "none";
    }
  }

  _archiveButtonClicked() {
    if (!this._editing) {
      this.dispatchEvent(new CustomEvent('archived', {
        detail: this.injury
      }));
      const url = appurl + "/api/injuries/" + this.injury.id + "/archive";
      const fetchOptions = {
        method: "PUT"
      };
      store.dispatch(fetchResource(url, fetchOptions));
    }
  }

  _unarchiveButtonClicked() {
    if (!this._editing) {
      this.dispatchEvent(new CustomEvent('unarchived', {
        detail: this.injury
      }));
      const url = appurl + "/api/injuries/" + this.injury.id + "/unarchive";
      const fetchOptions = {
        method: "PUT"
      };
      store.dispatch(fetchResource(url, fetchOptions));
    }
  }

  _editButtonClicked() {
    this._editing = !this._editing;
  }

  _editTitle(event) {
    const formElement = this.shadowRoot.getElementById("injuryEdit");
    const jsonFormData = formDataToJson(formElement);
    this.dispatchEvent(new CustomEvent('title-changed', {
      detail: this.injury
    }));
    const url = appurl + "/api/injuries/" + this.injury.id + "/editTitle";
    const fetchOptions = {
      method: "PUT",
      headers: { "Content-Type": "application/json" },
      body: jsonFormData
    };
    store.dispatch(fetchResource(url, fetchOptions));
    this._editing = false;
    event.preventDefault();
  }

  updated(changedProps) {
    if (changedProps.has('_editing')) {
      const input = this.shadowRoot.getElementById("title-input");
      (this._editing) ? input.style.background = "var(--medium-sand-color)" : input.style.background = "transparent";
      const submit = this.shadowRoot.getElementById("submit-edit");
      (this._editing) ? submit.style.visibility = "visible" : submit.style.visibility = "hidden";
    }
  }
}

window.customElements.define('my-injury', MyInjury);
